
using System;
using NUnit.Framework;
using MySql.Data.MySqlClient;
using System.Data;

namespace Test.MySqlException
{
  [TestFixture()]
  public class TestStackTraces
  {
    [Test()]
    public void TestBogusStackTrace()
    {
      using (IDbConnection connection = CreateMySqlConnection())
      {
        using (IDbCommand command = connection.CreateCommand())
        {
          // Schema `test` contains one table in my test DB
          // `bar` has one column: `baz` INT
          
          // So - this SQL command is going to create a MySqlException
          command.CommandText = "SELECT foo FROM bar;";
          
          // Here comes that exception!
          command.ExecuteNonQuery();
          
          /* This is the stack trace I see with the stock MySql.Data.dll
           * 
           * Test.MySqlException.TestStackTraces.TestBogusStackTrace:
           * MySql.Data.MySqlClient.MySqlException : Unknown column 'foo' in 'field list'
           * at MySql.Data.MySqlClient.MySqlStream.ReadPacket () [0x00000]
           * at MySql.Data.MySqlClient.NativeDriver.GetResult (System.Int32& affectedRow, System.Int32& insertedId) [0x00000]
           */
          
          
          /* With my rebuilt MySql.Data.dll (using the modified NativeDriver.cs) this is the stack trace I see instead:
           * 
           * Test.MySqlException.TestStackTraces.TestBogusStackTrace:
           * MySql.Data.MySqlClient.MySqlException : Fatal error encountered during command execution.
           *   ----> MySql.Data.MySqlClient.MySqlException : Fatal error encountered attempting to read the resultset.
           *   ----> MySql.Data.MySqlClient.MySqlException : Rethrown exception
           *   ----> MySql.Data.MySqlClient.MySqlException : Unknown column 'foo' in 'field list'
           * at MySql.Data.MySqlClient.MySqlCommand.ExecuteReader (CommandBehavior behavior) [0x00000]
           * at MySql.Data.MySqlClient.MySqlCommand.ExecuteReader () [0x00000]
           * at MySql.Data.MySqlClient.MySqlCommand.ExecuteNonQuery () [0x00000]
           * at Test.MySqlException.TestStackTraces.TestBogusStackTrace () [0x00000]
           * at (wrapper managed-to-native) System.Reflection.MonoMethod:InternalInvoke (object,object[],System.Exception&)
           * at System.Reflection.MonoMethod.Invoke (System.Object obj, BindingFlags invokeAttr, System.Reflection.Binder binder, System.Object[] parameters, System.Globalization.CultureInfo culture) [0x00000]
           * --MySqlException
           * at MySql.Data.MySqlClient.MySqlDataReader.NextResult () [0x00000]
           * at (wrapper remoting-invoke-with-check) MySql.Data.MySqlClient.MySqlDataReader:NextResult ()
           * at MySql.Data.MySqlClient.MySqlCommand.ExecuteReader (CommandBehavior behavior) [0x00000]
           * --MySqlException
           * at MySql.Data.MySqlClient.NativeDriver.GetResult (System.Int32& affectedRow, System.Int32& insertedId) [0x00000]
           * at MySql.Data.MySqlClient.Driver.GetResult (Int32 statementId, System.Int32& affectedRows, System.Int32& insertedId) [0x00000]
           * at MySql.Data.MySqlClient.Driver.NextResult (Int32 statementId) [0x00000]
           * at MySql.Data.MySqlClient.MySqlDataReader.NextResult () [0x00000]
           * --MySqlException
           * at MySql.Data.MySqlClient.MySqlStream.ReadPacket () [0x00000]
           * at MySql.Data.MySqlClient.NativeDriver.GetResult (System.Int32& affectedRow, System.Int32& insertedId) [0x00000]
           */
        }
      }
    }
    
    private IDbConnection CreateMySqlConnection()
    {
      MySqlConnection output;
      string connString = @"Server=127.0.0.1;
                            Port=3306;
                            Database=test;
                            User ID=root;
                            Pooling=false;
                            Allow User Variables=True";
      
      output = new MySqlConnection(connString);
      output.Open();
      return output;
    }
  }
}
